﻿using Electric.Domain.Repositories;
using Electric.EntityFrameworkCore.Repository;

using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;

namespace Electric.EntityFrameworkCore.DependencyInjection
{
    public static class ServiceCollectionExtensions
    {
        /// <summary>
        /// EntityFrameworkCore注入
        /// </summary>
        /// <param name="services"></param>
        /// <param name="dbType"></param>
        /// <param name="connection"></param>
        /// <param name="assemblyName"></param>
        public static void AddEntityFrameworkCore(this IServiceCollection services, DbType dbType, string connection, string? assemblyName = null)
        {
            #region 初始化数据库

            // 如果当前程序集没有Migrations文件夹且没有快照时执行如下两步进行初始化数据库
            //1.输入：Add-Migration InitDB，并回车。其中：Add-Migration为迁移命令，InitDB：这个是迁移的名称，可以根据自己自定义。
            //2.在程序包管理器控制台，输入命令：Update-Database，并回车。

            #endregion 初始化数据库

            //Repository注入
            var assemblies = AppDomain.CurrentDomain.GetAssemblies();
            foreach (var assembly in assemblies)
            {
                //获取继承IRepository并且不是泛型的类
                List<Type> types = assembly.GetTypes()
                .Where(t => t.IsClass && !t.IsGenericType && t.GetInterfaces().Contains(typeof(IRepository)))
                .ToList();

                types.ForEach(impl =>
                {
                    //获取该类所继承的所有接口
                    Type[] interfaces = impl.GetInterfaces();
                    interfaces.ToList().ForEach(i =>
                    {
                        services.AddScoped(i, impl);
                    });
                });
            }

            //数据库上下文注入
            services.AddDbContext<ApplicationDbContext>(options =>
            {
                //启用的数据库类型
                switch (dbType)
                {
                    case DbType.SqlServer:
                        if (assemblyName == null)
                        {
                            options.UseSqlServer(connection);
                        }
                        else
                        {
                            //AddEntityFrameworkCore增加一个参数：assemblyName，用来指定迁移程序集的名称。
                            options.UseSqlServer(connection, b => b.MigrationsAssembly(assemblyName));
                        }
                        break;

                    case DbType.MySql:
                        if (assemblyName == null)
                        {
                            options.UseMySql(connection, ServerVersion.AutoDetect(connection));
                        }
                        else
                        {
                            //AddEntityFrameworkCore增加一个参数：assemblyName，用来指定迁移程序集的名称。
                            options.UseMySql(connection, ServerVersion.AutoDetect(connection), b => b.MigrationsAssembly(assemblyName));
                        }
                        break;
                }
            });
        }
    }
}